@@ -36,12 +36,10 @@ class TestDecompressionBomb:
         Image.MAX_IMAGE_PIXELS = 128 * 128 - 1
         assert Image.MAX_IMAGE_PIXELS == 128 * 128 - 1
 
-        def open():
+        with pytest.warns(Image.DecompressionBombWarning):
             with Image.open(TEST_FILE):
                 pass
 
-        pytest.warns(Image.DecompressionBombWarning, open)
-
     def test_exception(self):
         # Set limit to trigger exception on the test file
         Image.MAX_IMAGE_PIXELS = 64 * 128 - 1
@@ -51,7 +49,6 @@ class TestDecompressionBomb:
             with Image.open(TEST_FILE):
                 pass
 
-    @pytest.mark.xfail(reason="different exception")
     def test_exception_ico(self):
         with pytest.raises(Image.DecompressionBombError):
             with Image.open("Tests/images/decompression_bomb.ico"):
@@ -62,6 +59,20 @@ class TestDecompressionBomb:
             with Image.open("Tests/images/decompression_bomb.gif"):
                 pass
 
+    def test_exception_gif_extents(self):
+        with Image.open("Tests/images/decompression_bomb_extents.gif") as im:
+            with pytest.raises(Image.DecompressionBombError):
+                im.seek(1)
+
+    def test_exception_gif_zero_width(self):
+        # Set limit to trigger exception on the test file
+        Image.MAX_IMAGE_PIXELS = 4 * 64 * 128
+        assert Image.MAX_IMAGE_PIXELS == 4 * 64 * 128
+
+        with pytest.raises(Image.DecompressionBombError):
+            with Image.open("Tests/images/zero_width.gif"):
+                pass
+
     def test_exception_bmp(self):
         with pytest.raises(Image.DecompressionBombError):
             with Image.open("Tests/images/bmp/b/reallybig.bmp"):
@@ -70,20 +81,21 @@ class TestDecompressionBomb:
 
 class TestDecompressionCrop:
     @classmethod
-    def setup_class(self):
+    def setup_class(cls):
         width, height = 128, 128
         Image.MAX_IMAGE_PIXELS = height * width * 4 - 1
 
     @classmethod
-    def teardown_class(self):
+    def teardown_class(cls):
         Image.MAX_IMAGE_PIXELS = ORIGINAL_LIMIT
 
-    def testEnlargeCrop(self):
+    def test_enlarge_crop(self):
         # Crops can extend the extents, therefore we should have the
         # same decompression bomb warnings on them.
         with hopper() as src:
             box = (0, 0, src.width * 2, src.height * 2)
-            pytest.warns(Image.DecompressionBombWarning, src.crop, box)
+            with pytest.warns(Image.DecompressionBombWarning):
+                src.crop(box)
 
     def test_crop_decompression_checks(self):
         im = Image.new("RGB", (100, 100))
@@ -91,7 +103,8 @@ class TestDecompressionCrop:
         for value in ((-9999, -9999, -9990, -9990), (-999, -999, -990, -990)):
             assert im.crop(value).size == (9, 9)
 
-        pytest.warns(Image.DecompressionBombWarning, im.crop, (-160, -160, 99, 99))
+        with pytest.warns(Image.DecompressionBombWarning):
+            im.crop((-160, -160, 99, 99))
 
         with pytest.raises(Image.DecompressionBombError):
             im.crop((-99909, -99990, 99999, 99999))
